<?php

// Author: oxana.smirnova@quark.lu.se

require_once('headfoot.inc');
require_once('lmtable.inc');
require_once('cnvtime.inc');
require_once('comfun.inc');
require_once('toreload.inc');
require_once('recursive_giis_info.inc');
require('ldap_purge.inc');

// Setting up the page itself

$toppage = new LmDoc("attlist");

/**
 * @return int
 * @param bdn string
 * @param owner string
 * @desc Retrieves the list of user's jobs
 */
function do_attlist ($bdn,$attribute)
{
  /**
   * FUNCTION DO_ATTRLIST (string bdn, string attribute)
   * Retrieves the user jobs information from the NorduGrid
   * Uses LDAP functions of PHP
   * 
   * Author: O.Smirnova (June 2002)
   * inspired by the LDAPExplorer by T.Miao
   * 
   * input:
   * bdn (base DN, def. mds-vo-name=NordiGrid,o=Grid)
   * attribute
   * 
   * output:
   * an HTML table, containing values of a given attribute at each cluster
   */

  include('settings.inc');
  include('isattr.inc');

  $tlim = 20;
  $tout = 20;

  $attribute = rawurldecode($attribute);
  $engatt   = ($isattr[$attribute]) ? $isattr[$attribute] : $attribute;
  
  // Array defining the attributes to be returned
  
  $lim = array( "dn", $attribute );
  
  // ldapsearch filter string for the attribute
  
  $components = explode("-",$attribute);
  $object     = $components[0]."-".$components[1];
  $filter     = "(&(objectclass=$object)($attribute=*))";

  // Header table
  
  $toptable = new LmTableTop("","$engatt:");

  $gentries = recursive_giis_info($giislist,"cluster",0);
  $nc       = count($gentries);
  if ( !$nc ) {
    // NO SITES FOUND!
    echo "<BR><font color=red><B>Information System is empty</B><br>(None of the top level GIIS returned information)</font>\n";
    return 1;
  }

  $dsarray = array ();
  $hnarray = array ();
  $sitetag = array (); /* a tag to skip duplicated entries */

  for ( $k = 0; $k < $nc; $k++ ) {
    $clhost = $gentries[$k]["host"];
    $clport = $gentries[$k]["port"];

    $clconn = ldap_connect($clhost,$clport);
    if ( $clconn && !$sitetag[$clhost] ) {
      array_push($dsarray,$clconn);
      $sitetag[$clhost] = 1; /* filtering tag */
    }
  }

  $nhosts = count($dsarray);
  if ( !$nhosts ) {
    // NO SITES REPLY...
    echo "<BR><font color=red><B>Information System is empty</B><br>(None of the GRIS returned connection)</font>\n";
    return 1;
  }

  // Search all clusters for (a) allowed queues and (b) for user jobs

  $srarray = @ldap_search($dsarray,DN_LOCAL,$filter,$lim,0,0,$tlim,LDAP_DEREF_NEVER,$tout);
  
  // HTML table initialisation
  $jtable  = new LmTableSp("attlist");
  $rowcont = array();
         
  for ( $ids = 0; $ids < $nhosts; $ids++ ) {

    $sr    = $srarray[$ids];
    $dst   = $dsarray[$ids];
    if ($dst && $sr) {
       
      // If search returned, check that there are valid entries
       
      $nmatch = ldap_count_entries($dst,$sr);
      if ($nmatch > 0) {
        // If there are valid entries, tabulate results
         
        $allentries = ldap_get_entries($dst,$sr);
	$entries    = ldap_purge($allentries);
        $nclus      = $entries["count"];
        
        for ($i=0; $i<$nclus; $i++) {

	  $cluster  = "N/A";
	  $queue    = "N/A";
	  $job      = "N/A";

	  $dnparts = ldap_explode_dn($entries[$i]["dn"],0);
	  foreach ($dnparts as $part) {
	    $pair = explode("=",$part);
	    switch ( $pair[0] ) {
	    case CLU_NAME:
	      $cluster = $pair[1];
	      break;
	    case QUE_NAME:
	      $queue   = $pair[1];
	      break;
	    case JOB_GLID:
	      $job     = $pair[1];
              $encjob  = rawurlencode($entries[$i]["dn"]);
	      break;
	    }
	  }

	  switch ( $object ) {
	  case OBJ_CLUS:
	    $resource  = "Cluster <i>$cluster</i>";
            $winstring = popup("clusdes.php?host=$cluster&port=2135",700,620,1);
	    break;
	  case OBJ_QUEU:
	    $resource  = "Cluster <i>$cluster</i>, queue <i>$queue</i>";
	    $winstring = popup("quelist.php?host=$cluster&port=2135&qname=$queue",750,430,6);
	    break;
	  case OBJ_AJOB:
	    $resource  = "Job <i>$job</i>";
	    $winstring = popup("jobstat.php?host=$cluster&port=2135&status=&jobdn=$encjob",750,430,4);
	    break;
	  }

          $rowcont[0] = "<a href=\"$winstring\">$resource</a>";
	  $rowcont[1] = "&nbsp;";

	  $vcount   = $entries[$i][$attribute]["count"];        
	  if ($vcount == 0) $jtable->addrow($rowcont);
        
	  for ($j=0; $j<$vcount; $j++) {
          $curval = ($entries[$i][$attribute][$j]) ? $entries[$i][$attribute][$j] : "N/A";
          
          // Some time-stamp readability adjustment
          if (substr(strrchr($attribute, "-"), 1) == "sessiondirerasetime" ||
              substr(strrchr($attribute, "-"), 1) == "submissiontime"      ||
              substr($attribute,0,9)              == "Mds-valid"
             ) $curval=cnvtime($curval);

          $rowcont[1] = $curval;
          $jtable->addrow($rowcont);
          $rowcont[0] = "&nbsp;";
          
        }
          $rowcont = array();
        }
      }
      //      else {
      //        echo "<BR><font color=red><B>No matching attributes found</B></font>\n";
      //        return 1;
      //      }
    }
    //    else {
    //      echo "<BR><font color=red><B>No information available</B></font>\n";
    //      return 1;
    //    }
    ldap_free_result($sr);
  }
  $jtable->close();
  return 0;
}

$attribute = $_GET["attribute"];
do_attlist(DN_GLOBL,$attribute);

// Done

$toppage->close();

?>
